@@ -244,6 +244,38 @@ class BNGLModel(Model):
                 return act_type, match.group(1)
         return None
 
+    def find_t_length(self):
+        text_list = []
+        fileText = self.bngl_file_text.split('\n')
+        timeDict = {}
+        
+                # have to clear any possible commented out lines that have n_steps in them
+        text_list = []
+        final_text = {}
+        for i in fileText:
+            if '#' in i:
+                pass
+            else:
+                text_list.append(i)
+            for i in text_list:
+                if 'simulate' not in i:
+                    pass
+                else:
+                    stuffStart = i.rfind('suffix=>')
+                    stuff = i[stuffStart:].split(',')
+                    for suf in stuff:
+                        if 'suffix' in suf:
+                            stuff = suf.split('=>')
+                            stuff = stuff[1]
+                            start = i.rfind('n_steps=>')
+                            text = i[start:]
+                            new_text = re.findall(r'[ 0-9]+', text)
+                            time = int(new_text[0])
+                            stuff = stuff.replace('"', '')
+                            timeDict[stuff] = time
+                
+        return timeDict                
+
     def copy_with_param_set(self, pset):
         """
         Returns a copy of this model containing the specified parameter set.
@@ -430,6 +462,7 @@ class BNGLModel(Model):
         return result
 
 
+
 class NetModel(BNGLModel):
     def __init__(self, name, acts, suffs, mutants, ls=None, nf=None):
         self.name = name
@@ -741,6 +774,7 @@ class TimeCourse(Action):
         self.model = ''
         self.suffix = 'time_course'
         self.method = 'ode'
+    
 
         # Transfer all the keys in the dict to my attributes of the same name
         for k in d:
@@ -772,7 +806,6 @@ class TimeCourse(Action):
         self.stepnumber = int(np.round(self.time/self.step))
         self.bng_codeword = 'simulate'
 
-
 class ParamScan(Action):
 
     def __init__(self, d):
@@ -1060,6 +1093,19 @@ class FreeParameter(object):
         else:
             return self.set_value(self.value + summand, reflect)
 
+    def add_norm(self, summand, reflect=True):
+        """
+        Adds a value to the existing value and returns a new FreeParameter instance. This version of add does
+        not consider the space that the value is in and just sums them
+
+        :param summand: Value to add
+        :return:
+        """
+        if self.value is None:
+            logger.error('Cannot add to FreeParameter with "None" value')
+        
+        return self.set_value(self.value + summand, reflect)
+
     def add_rand(self, lb, ub, reflect=True):
         """
         Like FreeParameter.add but instead adds a uniformly distributed random value according to the
